# Generated by Django 2.2.6 on 2019-10-25 18:16

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('inputgooglesheets', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='spreadsheet',
            name='enabled',
            field=models.BooleanField(blank=True, default=True, verbose_name='Enabled'),
        ),
        migrations.AlterField(
            model_name='spreadsheet',
            name='omit_first_row',
            field=models.BooleanField(blank=True, default=True, verbose_name='Omit first row'),
        ),
    ]



import pytz
import datetime
from google.auth.credentials import Credentials as BaseCredentials

class Credentials(BaseCredentials):
    def __init__(self, token_obj):
        self.token_obj = token_obj
        self._update_from_token_obj()

    def _update_from_token_obj(self):
        self.token = self.token_obj.access_token
        utc = pytz.timezone('UTC')
        self.expiry = self.token_obj.expiration().astimezone(utc).replace(tzinfo=None)

    def refresh(self, *args):
        self.token_obj.do_refresh_token()
        self._update_from_token_obj()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from django.utils.six import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _, ugettext
from django.db import models
from oauth_clients.models import Client, AccessToken as BaseAccessToken
from inputflow.models import InputSettings


@python_2_unicode_compatible
class Settings(models.Model):
    oauth_client = models.ForeignKey(Client, verbose_name=_("Oauth client"),
        blank=True, null=True, default=None, related_name='google_sheets_clients',
        on_delete=models.SET_NULL)

    class Meta:
        verbose_name = _("Settings")
        verbose_name_plural = _("Settings")

    def __str__(self):
        return ugettext("Settings")

class AccessTokenManager(models.Manager):
    def get_queryset(self):
        result = super(AccessTokenManager, self).get_queryset()
        settings = Settings.objects.first()
        if settings is None or settings.oauth_client is None:
            return result.none()
        return result.filter(client=settings.oauth_client)

@python_2_unicode_compatible
class AccessToken(BaseAccessToken):
    objects = AccessTokenManager()

    class Meta:
        verbose_name = _("Access token")
        verbose_name_plural = _("Access tokens")
        proxy = True

    def __str__(self):
        return '{} ({})'.format(self.username, self.get_scope())

    def get_scope(self):
        result = self.scope.split()
        result = [scope.split('/')[-1] for scope in result]
        return ', '.join(result)


@python_2_unicode_compatible
class Spreadsheet(models.Model):
    access_token = models.ForeignKey(AccessToken, verbose_name=_("Access token"),
        related_name='available_google_sheets', on_delete=models.CASCADE)
    url = models.URLField(_("URL"), max_length=500)
    name = models.CharField(_("Name"), max_length=250)
    spreadsheet_name = models.CharField(_("Spreadsheet name"), max_length=250)
    uid = models.CharField(_("UID"), max_length=250)
    sheet_index = models.PositiveIntegerField(_("Sheet index"), blank=True, default=0)
    sheet_name = models.CharField(_("Sheet name"), max_length=250, blank=True, default='')

    omit_first_row = models.BooleanField(_("Omit first row"), blank=True,
        default=True)
    last_imported_row = models.PositiveIntegerField(_("Last imported row"),
        blank=True, null=True, default=None)
    last_imported_date = models.DateTimeField(_("Last imported date"),
        blank=True, null=True, default=None)
    enabled = models.BooleanField(_("Enabled"), blank=True, default=True)
    automatically_imported = models.BooleanField(_("Automatically imported"), blank=True,
        default=True)
    
    input_settings = models.ForeignKey(InputSettings, verbose_name=_("Input settings"),
        blank=True, null=True, default=None, on_delete=models.PROTECT,
        related_name='google_spreadsheets')

    class Meta:
        verbose_name = _("Spredsheet")
        verbose_name_plural = _("Spreadsheets")
        unique_together = [('access_token', 'url'),]

    def __str__(self):
        return self.name

    def reset_importing(self):
        self.last_imported_date = None
        self.last_imported_row = None
        self.save()

    def get_start_row_number(self):
        return 2 if self.omit_first_row else 1

    def get_next_row_number(self):
        if self.last_imported_row is None:
            return self.get_start_row_number()
        return self.last_imported_row + 1



from django.conf import settings


CONNECTION = None

def _get_connection():
    import redis_lock
    global CONNECTION
    if CONNECTION is None:
        CONNECTION = redis_lock.StrictRedis(**getattr(settings, 'REDIS_LOCK_CONN', {}))
    return CONNECTION


def lock_creator(key):
    "Distributed lock with redis"
    import redis_lock
    return redis_lock.Lock(_get_connection(), key)



# Generated by Django 2.2.6 on 2019-10-26 02:21

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('inputgooglesheets', '0002_auto_20191025_1516'),
    ]

    operations = [
        migrations.AddField(
            model_name='spreadsheet',
            name='automatically_imported',
            field=models.BooleanField(blank=True, default=True, verbose_name='Automatically imported'),
        ),
    ]



from __future__ import absolute_import, unicode_literals
import logging
import functools
from celery import shared_task
from django.conf import settings
from django.utils.module_loading import import_string


logger = logging.getLogger(__name__)


LOCK_KEY = '744ed748-7685-41ca-a714-f0bfb6db0b80:{}'


def get_lock(key):
    "Returns a lock object configured with the key that must be locked."
    lock_key = LOCK_KEY.format(key)
    lock_creator = import_string(settings.DISTRIBUTED_LOCK_CREATOR)
    return lock_creator(lock_key)


def get_spreadsheet_and_lock(pk):
    """Returns a spreadsheet and a lock configured with the key of the
    spreadsheet. If the spreadshed is disabled or does not exist, writes
    a warn log informing it and returns a tuple of two `None` values.

    Parameters
    ----------
    pk: int
        The pk value of the spreadsheet to return.

    Returns
    -------
    spreadsheet: inputgooglesheets.models.Spreadsheet
        Spreadsheet.
    lock: any
        A lock object configured for the speadsheet pk.
    """
    from .models import Spreadsheet
    try:
        result = Spreadsheet.objects.get(pk=pk)
        if not result.enabled:
            logger.warn("The spreadsheet '{}' is disabled.".format(pk))
            return None, None
        return result, get_lock(pk)
    except Spreadsheet.DoesNotExist:
        logger.warn("Spreadsheet '{}' does not exist.".format(pk))
        return None, None


def with_spreadsheet_and_lock(fn):
    def spreadsheet_and_lock(pk):
        spreadsheet, lock = get_spreadsheet_and_lock(pk)
        if lock and lock.acquire(blocking=False):
            try:
                return fn(spreadsheet)
            finally:
                lock.release()
        else:
            logger.warn("Already importing spreadsheet '{}'. Omiting import.".format(pk))
    return functools.update_wrapper(spreadsheet_and_lock, fn)


@shared_task
@with_spreadsheet_and_lock
def start_import(spreadsheet):
    from .utils import Utils
    count = Utils.start_import(spreadsheet)
    logger.info("Imported spreadsheet '{}'. {} rows imported".format(spreadsheet.pk, count))


@shared_task
@with_spreadsheet_and_lock
def continue_import(spreadsheet):
    from .utils import Utils
    count = Utils.continue_import(spreadsheet)
    logger.info("Finished importing spreadsheet '{}': {} rows imported".format(spreadsheet.pk, count))


@shared_task
def import_all_google_spreadsheets():
    from .models import Spreadsheet
    lock = get_lock('')
    if lock.acquire(blocking=False):
        try:
            for spreadsheet in Spreadsheet.objects.filter(enabled=True, automatically_imported=True):
                continue_import.delay(spreadsheet.pk)
        finally:
            lock.release()



import sys
import string
from collections import OrderedDict
import json
import hashlib
import requests
import pygsheets.client
from django.utils.six.moves.urllib.parse import urlencode
from django.utils.timezone import now
from inputflow.models import Input
from . import models
from . import credentials

class Utils:
    @staticmethod
    def fetch_token_info(token):
        """
        update token user_id and username if the token belongs to the oauth client
        selected in the settings object.
        """
        settings = models.Settings.objects.filter(oauth_client=token.client).first()
        if settings is not None:
            url = "https://www.googleapis.com/oauth2/v1/tokeninfo?access_token={}"
            url = url.format(token.access_token)
            response = requests.get(url)
            if response.status_code == 200:
                info = response.json()
                token.user_id = info['user_id']
                token.username = info['email']

    @staticmethod
    def get_spreadsheet(spreadsheet_obj):
        creds = credentials.Credentials(spreadsheet_obj.access_token)
        client = pygsheets.client.Client(creds)
        return client.open_by_url(spreadsheet_obj.url)

    @classmethod
    def get_worksheet(cls, spreadsheet_obj):
        spreadsheet = cls.get_spreadsheet(spreadsheet_obj)
        return spreadsheet.worksheets()[spreadsheet_obj.sheet_index]

    @staticmethod
    def worksheet_row_to_dict(row):
        non_empty = [col for col in row if col != '']
        if len(non_empty) == 0:
            return None
        cols = ['COL${}'.format(letter) for letter in string.ascii_uppercase]
        encode = lambda value: value.encode('utf-8') if sys.version_info.major == 2 else value
        result = [(col, encode(value)) for col, value in zip(cols, row) if value != '']
        result = OrderedDict(result)
        content = json.dumps(result)
        sha1 = hashlib.sha1(content.encode('utf-8'))
        result['_content_hash'] = sha1.hexdigest()
        return result

    @staticmethod
    def save_worksheet_row_to_input(spreadsheet_obj, row):
        encoded_row = urlencode(row)
        input_row = Input()
        input_row.settings = spreadsheet_obj.input_settings
        input_row.internal_source = True
        input_row.format = 'form'
        input_row.raw_content = encoded_row
        input_row.save()
        spreadsheet_obj.last_imported_row = row['row']
        spreadsheet_obj.last_imported_date = now()
        spreadsheet_obj.save()
        input_row.notify()

    @classmethod
    def import_to_input_flow(cls, spreadsheet_obj, max_count=50, start_row=None):
        if spreadsheet_obj.input_settings is None:
            return
        row_index = start_row or spreadsheet_obj.get_next_row_number()
        worksheet = cls.get_worksheet(spreadsheet_obj)
        count = 0
        imported = True
        while imported:
            values = worksheet.get_values((row_index, 1), (row_index+max_count-1, worksheet.cols))
            imported = False
            for row in values:
                row = cls.worksheet_row_to_dict(row)
                if row is None:
                    break
                row['row'] = row_index
                cls.save_worksheet_row_to_input(spreadsheet_obj, row)
                row_index += 1
                count += 1
                imported = True
        return count

    @classmethod
    def start_import(cls, spreadsheet_obj, max_count=50):
        return cls.import_to_input_flow(spreadsheet_obj, max_count=max_count,
            start_row=spreadsheet_obj.get_start_row_number())

    @classmethod
    def continue_import(cls, spreadsheet_obj, max_count=50):
        return cls.import_to_input_flow(spreadsheet_obj, max_count=max_count,
            start_row=spreadsheet_obj.get_next_row_number())



# -*- coding: utf-8 -*-
# Generated by Django 1.11.25 on 2019-10-24 05:25
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('inputflow', '0003_auto_20191011_0015'),
        ('oauth_clients', '0006_accesstoken_scope'),
    ]

    operations = [
        migrations.CreateModel(
            name='Settings',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('oauth_client', models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='google_sheets_clients', to='oauth_clients.Client', verbose_name='Oauth client')),
            ],
            options={
                'verbose_name': 'Settings',
                'verbose_name_plural': 'Settings',
            },
        ),
        migrations.CreateModel(
            name='Spreadsheet',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('url', models.URLField(max_length=500, verbose_name='URL')),
                ('name', models.CharField(max_length=250, verbose_name='Name')),
                ('spreadsheet_name', models.CharField(max_length=250, verbose_name='Spreadsheet name')),
                ('uid', models.CharField(max_length=250, verbose_name='UID')),
                ('sheet_index', models.PositiveIntegerField(blank=True, default=0, verbose_name='Sheet index')),
                ('sheet_name', models.CharField(blank=True, default='', max_length=250, verbose_name='Sheet name')),
                ('omit_first_row', models.BooleanField(default=True, verbose_name='Omit first row')),
                ('last_imported_row', models.PositiveIntegerField(blank=True, default=None, null=True, verbose_name='Last imported row')),
                ('last_imported_date', models.DateTimeField(blank=True, default=None, null=True, verbose_name='Last imported date')),
            ],
            options={
                'verbose_name': 'Spredsheet',
                'verbose_name_plural': 'Spreadsheets',
            },
        ),
        migrations.CreateModel(
            name='AccessToken',
            fields=[
            ],
            options={
                'verbose_name': 'Access token',
                'proxy': True,
                'verbose_name_plural': 'Access tokens',
                'indexes': [],
            },
            bases=('oauth_clients.accesstoken',),
        ),
        migrations.AddField(
            model_name='spreadsheet',
            name='access_token',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='available_google_sheets', to='inputgooglesheets.AccessToken', verbose_name='Access token'),
        ),
        migrations.AddField(
            model_name='spreadsheet',
            name='input_settings',
            field=models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='google_spreadsheets', to='inputflow.InputSettings', verbose_name='Input settings'),
        ),
        migrations.AlterUniqueTogether(
            name='spreadsheet',
            unique_together=set([('access_token', 'url')]),
        ),
    ]
